@@ -5,20 +5,19 @@
 # The first argument of all these methods is node
 
 
-
 #' Print a \code{Node} in a human-readable fashion.
-#'  
+#'
 #' @param x The Node
 #' @param ... Node attributes to be printed. Can be either a character (i.e. the name of a Node field),
-#' a Node method, or a function taking a Node as a single argument. See \code{Get} for details on 
+#' a Node method, or a function taking a Node as a single argument. See \code{Get} for details on
 #' the meaning of \code{attribute}.
 #' @param pruneMethod The method used to prune for printing. If NULL, the entire tree is displayed. If
 #' "simple", then only the first \code{limit} nodes are displayed. If "dist", then Nodes are removed
 #' everywhere in the tree, according to their level.
-#' @param limit The maximum number of nodes to print. Can be \code{NULL} if the 
+#' @param limit The maximum number of nodes to print. Can be \code{NULL} if the
 #' entire tree should be printed.
-#' 
-#' 
+#'
+#'
 #' @examples
 #' data(acme)
 #' print(acme, "cost", "p")
@@ -31,9 +30,9 @@ print.Node <- function(x, ..., pruneMethod = c("simple", "dist", NULL), limit =
   pruneMethod <- pruneMethod[1]
   if (length(pruneMethod) > 0 && length(limit) > 0) {
     if (pruneMethod == "simple") {
-      x <- PrintPruneSimple(x, limit = limit)    
+      x <- PrintPruneSimple(x, limit = limit)
     } else if (pruneMethod == "dist") {
-      x <- PrintPruneDist(x, limit = limit)    
+      x <- PrintPruneDist(x, limit = limit)
     } else {
       stop (paste0("Unknown pruneMethod ", pruneMethod, "!"))
     }
@@ -48,50 +47,61 @@ print.Node <- function(x, ..., pruneMethod = c("simple", "dist", NULL), limit =
 }
 
 
-#' Aggregate child values of a \code{Node}, standalone or in traversal.
-#' 
-#' The \code{Aggregate} method lets you fetch an attribute from a \code{Node}'s children, and then aggregate it.
-#' For example, you can aggregate cost by summing costs of child \code{Nodes}. This is especially useful in the 
+
+
+
+#' Aggregate child values of a \code{Node}, recursively.
+#'
+#' The \code{Aggregate} method lets you fetch an attribute from a \code{Node}'s children, and then aggregate them
+#' using \code{aggFun}. For example, you can aggregate cost by summing costs of child \code{Nodes}. This is especially useful in the
 #' context of tree traversal, when using post-order traversal mode.
-#' 
-#' As with \code{\link{Get}}, the attribute can be a field, a method or a function. If it is a field, and if
-#' the node contains the attribute, its value is returned. 
-#' Otherwise, \code{aggFun(Aggregate(children, ...))} is called. In that case, 
-#' the attribute must be set on the leaf.
-#' 
+#'
+#' As with \code{\link{Get}}, the attribute can be a field, a method or a function. If the attribute on a child 
+#' is \code{NULL}, \code{Aggregate} is called recursively on its children. 
+#'
 #' @param node the \code{Node} on which to aggregate
 #' @param aggFun the aggregation function to be applied to the children's \code{attributes}
 #' @param ... any arguments to be passed on to attribute (in case it's a function)
 #'
 #' @inheritParams Get
-#'   
+#'
 #' @examples
 #' data(acme)
-#' 
+#'
 #' #Aggregate on a field
 #' Aggregate(acme, "cost", sum)
 #' 
+#' #This is the same as:
+#' HomeRolledAggregate <- function(node) {
+#'   sum(sapply(node$children, function(child) {
+#'     if (!is.null(child$cost)) child$cost
+#'     else HomeRolledAggregate(child)
+#'   }))
+#' }
+#' HomeRolledAggregate(acme)
+#'
 #' #Aggregate using Get
 #' print(acme, "cost", minCost = acme$Get(Aggregate, "cost", min))
-#' 
+#'
 #' #use Aggregate with a function:
-#' acme$Do(function(x) x$expectedCost <- Aggregate(x, 
-#'                                                 function(x) x$cost * x$p, 
-#'                                                 sum)
-#'        , traversal = "post-order")
-#'   
+#' Aggregate(acme, function(x) x$cost * x$p, sum)
+#'
+#' #cache values along the way
+#' acme$Do(function(x) x$cost <- Aggregate(x, "cost", sum), traversal = "post-order")
+#' acme$IT$cost
+#'
 #' @seealso \code{\link{Node}}
-#' 
+#'
 #' @export
-Aggregate = function(node, 
-                     attribute, 
-                     aggFun, 
+Aggregate = function(node,
+                     attribute,
+                     aggFun,
                      ...) {
-  
+
   if("cacheAttribute" %in% names(list(...))) stop("cacheAttribute not supported anymore! Please use Do instead.")
-  
+
   if (isLeaf(node)) return ( GetAttribute(node, attribute, ...) )
-  values <- sapply(node$children, 
+  values <- sapply(node$children,
                    function(x) {
                      v <- GetAttribute(x, attribute, format = identity, ...)
                      if (length(v) > 0 && !is.na(v)) return(v)
@@ -103,21 +113,21 @@ Aggregate = function(node,
 }
 
 #' Cumulate values among siblings
-#' 
+#'
 #' For example, you can sum up values of siblings before
 #' this \code{Node}.
-#' 
+#'
 #' @param node The node on which we want to cumulate
-#' 
+#'
 #' @inheritParams Aggregate
 #' @inheritParams Get
-#' 
+#'
 #' @examples
 #' data(acme)
 #' acme$Do(function(x) x$cost <- Aggregate(x, "cost", sum), traversal = "post-order")
 #' acme$Do(function(x) x$cumCost <- Cumulate(x, "cost", sum))
 #' print(acme, "cost", "cumCost")
-#' 
+#'
 #' @export
 Cumulate = function(node, attribute, aggFun, ...) {
   if ("cacheAttribute" %in% names(list(...))) stop("cacheAttribute not supported anymore! Please use Do instead.")
@@ -125,38 +135,38 @@ Cumulate = function(node, attribute, aggFun, ...) {
   pos <- node$position
   nodes <- node$parent$children[1:pos]
   res <- aggFun(Get(nodes, attribute, format = identity))
-  
+
   return (res)
 }
 
 #' Clone a tree (creates a deep copy)
-#' 
+#'
 #' The method also clones object attributes (such as the formatters), if desired.
 #' If the method is called on a non-root, then the parent relationship is not cloned,
 #' and the resulting \code{\link{Node}} will be a root.
-#' 
+#'
 #' @param node the root node of the tree or sub-tree to clone
-#' @param attributes if FALSE, then R class attributes (e.g. formatters and grViz styles) 
+#' @param attributes if FALSE, then R class attributes (e.g. formatters and grViz styles)
 #' are not cloned. This makes the method faster.
 #' @return the clone of the tree or sub-tree
-#' 
+#'
 #' @examples
 #' data(acme)
 #' acmeClone <- Clone(acme)
 #' acmeClone$name <- "New Acme"
 #' # acmeClone does not point to the same reference object anymore:
 #' acme$name
-#' 
+#'
 #' #cloning a subtree
 #' data(acme)
 #' itClone <- Clone(acme$IT)
 #' itClone$isRoot
-#' 
-#' 
+#'
+#'
 #' @inheritParams Prune
-#' 
+#'
 #' @seealso SetFormat
-#' 
+#'
 #' @export
 Clone <- function(node, pruneFun = NULL, attributes = FALSE) {
   .Clone(node, pruneFun, attributes)
@@ -184,47 +194,89 @@ Clone <- function(node, pruneFun = NULL, attributes = FALSE) {
 }
 
 
+#' Navigate to another node by relative path.
+#' 
+#' @usage #node$Navigate(path)
+#' Navigate(node, path)
+#' 
+#' @param node The starting \code{\link{Node}} to navigate
+#' @param path A string or a character vector describing the path to navigate
+#' 
+#' @details The \code{path} is always relative to the \code{node}. Navigation
+#' to the parent is defined by \code{..}, whereas navigation to a child
+#' is defined via the child's name.
+#' If path is provided as a string, then the navigation steps are separated
+#' by '/'.
+#'
+#' @examples 
+#' data(acme)
+#' acme$Research$Navigate("../IT/Outsource")
+#' acme$Research$Navigate(c("..", "IT", "Outsource"))
+#'
+#' @seealso \code{\link{Climb}}
+#'
+#' @export
+Navigate <- function(node, path) {
+  if (length(path) == 1) path <- strsplit(path, "/", fixed = TRUE)[[1]]
+  for (nxt in path) {
+    if (identical("..", nxt)) {
+      node <- node$parent
+    } else if (identical(".", nxt)) {
+      #don't do anything
+    } else {
+      node <- node[[nxt]]
+    }
+  }
+  return (node)
+}
+
+
 
 
 #' Climb a tree from parent to children, by provided criteria.
-#' 
-#' 
-#' This method lets you climb the tree, from crutch to crutch. On each \code{Node}, the 
+#'
+#'
+#' This method lets you climb the tree, from crutch to crutch. On each \code{Node}, the
 #' \code{Climb} finds the first child having attribute value equal to the the provided argument.
-#' 
+#'
 #' @usage #node$Climb(...)
 #' Climb(node, ...)
-#' 
-#' 
-#' @param node The root node of the tree or subtree to climb
+#'
+#'
+#' @param node The root \code{\link{Node}} of the tree or subtree to climb
 #' @param ... an attribute name to searched value pairlist. For brevity, you can also provide a character vector.
 #' @return the \code{Node} having path \code{...}, or \code{NULL} if such a path does not exist
-#' 
+#'
 #' @examples
 #' data(acme)
-#' acme$Climb('IT', 'Outsource')$name
-#' acme$Climb('IT')$Climb('Outsource')$name
 #' 
+#' #the following are all equivalent
+#' acme$Climb('IT', 'Outsource')
+#' acme$Climb(name = 'IT', name = 'Outsource')
+#' acme$Climb('IT')$Climb('Outsource')
+#' acme$Navigate(path = "IT/Outsource")
+#'
 #' acme$Climb(name = 'IT')
-#' 
+#'
 #' acme$Climb(position = c(2, 1))
 #' #or, equivalent:
 #' acme$Climb(position = 2, position = 1)
 #' acme$Climb(name = "IT", cost = 250000)
-#' 
+#'
 #' tree <- CreateRegularTree(5, 2)
 #' tree$Climb(c("1", "1"), position = c(2, 2))$path
 #'
 #' @seealso \code{\link{Node}}
+#' @seealso \code{\link{Navigate}}
 #'
 #' @export
 Climb <- function(node, ...) {
-  
+
   path <- list(...)
   if (length(path) == 0) {
     return (node)
   } else {
-    
+
     #convert args to standard
     #e.g. id = (3, 5), name = "myname"
     #to
@@ -235,19 +287,20 @@ Climb <- function(node, ...) {
     mpath <- NULL
     for (i in 1:length(path)) names(path[[i]]) <- rep(names(path)[i], length(path[[i]]))
     for (i in 1:length(path)) mpath <- c(mpath, as.list(path[[i]]))
-    
+
     attribute <- names(mpath)[[1]]
     if (length(attribute) == 0 || is.na(attribute) || nchar(attribute) == 0) attribute <- "name"
-    
-    
+
+
     value <- mpath[[1]]
     
-    getA <- Get(node$children, attribute)
-    child <- node$children[getA == value]
-    if(length(child) == 0) return (NULL)
-    
-    child <- child[[1]]
+    if (attribute == "name") child <- node[[value]]
+    else {
+      getA <- Get(node$children, attribute)
+      child <- node$children[getA == value][[1]]
+    }
     
+
     if (is.null(child)) {
       return (NULL)
     } else if (length(mpath) == 1) {
@@ -256,31 +309,31 @@ Climb <- function(node, ...) {
       return (do.call(Climb, c(node = child, mpath[-1])))
     }
   }
-  
+
 }
 
 
 
 #' Find a node by name in the (sub-)tree
-#' 
+#'
 #' Scans the entire sub-tree spanned by \code{node} and returns the first \code{\link{Node}}
 #' having the \code{name} specified. This is mainly useful for trees whose name is unique.
 #' If \code{\link{AreNamesUnique}} is \code{FALSE}, i.e. if there is more than one \code{Node}
-#' called \code{name} in the tree, then it is undefined which one will be returned. 
-#' Also note that this method is not particularly fast. See examples for a faster way to 
+#' called \code{name} in the tree, then it is undefined which one will be returned.
+#' Also note that this method is not particularly fast. See examples for a faster way to
 #' index large trees, if you need to do multiple searches. See \code{\link{Traverse}} if
 #' you need to find multiple \code{Nodes}.
-#' 
+#'
 #' @param node The root \code{Node} of the tree or sub-tree to search
 #' @param name The name of the \code{Node} to be returned
-#' 
+#'
 #' @return The first \code{Node} whose name matches, or \code{NULL} if no such \code{Node} is
 #' found.
-#' 
+#'
 #' @examples
 #' data(acme)
 #' acme$FindNode("Outsource")
-#' 
+#'
 #' #re-usable hashed index for multiple searches:
 #' if(!AreNamesUnique(acme)) stop("Hashed index works for unique names only!")
 #' trav <- Traverse(acme, "level")
@@ -290,10 +343,10 @@ Climb <- function(node, ...) {
 #' #nameIndex <- hash(trav)
 #' nameIndex$Outsource
 #' nameIndex$IT
-#' 
-#' 
+#'
+#'
 #' @seealso AreNamesUnique, Traverse
-#' 
+#'
 #' @export
 FindNode <- function(node, name) {
   trav <- Traverse(node, filterFun = function(x) x$name == name)
@@ -305,17 +358,17 @@ FindNode <- function(node, name) {
 
 
 #' Get an attribute from a Node.
-#' 
+#'
 #' @param node The \code{\link{Node}} from which the \code{attribute} should be fetched.
 #' @param nullAsNa If TRUE (the default), then NULL is returned as NA. Otherwise it is returned as NULL.
-#' 
-#' 
+#'
+#'
 #' @inheritParams Get
-#' 
+#'
 #' @examples
 #' data(acme)
 #' GetAttribute(acme$IT$Outsource, "cost")
-#' 
+#'
 #' @export
 GetAttribute <- function(node, attribute, ..., format = NULL, inheritFromAncestors = FALSE, nullAsNa = TRUE) {
   if (is.function(attribute)) {
@@ -332,41 +385,28 @@ GetAttribute <- function(node, attribute, ..., format = NULL, inheritFromAncesto
   } else {
     stop("attribute must be a function, the name of a public property, or the name of method")
   }
-  
-  if (length(v) == 0 && inheritFromAncestors && !node$isRoot) {
-    v <- GetAttribute(node$parent, attribute, 
-                      ..., 
+
+  if (is.null(v) && inheritFromAncestors && !node$isRoot) {
+    v <- GetAttribute(node$parent, attribute,
+                      ...,
                       inheritFromAncestors = TRUE,
                       nullAsNa = FALSE)
   }
-  
-  if (!nullAsNa && length(v) == 0) return (NULL)
-  if (length(v) == 0) v <- NA
-  if (length(v) == 0) v <- NA
-  
-  
-  
+
+  if (!nullAsNa && is.null(v)) return (NULL)
+  if (is.null(v)) v <- NA
+
+
   if (is.null(format) && !is.function(attribute)) {
     #get default formatter
     format <- GetObjectAttribute(node, "formatters")[[attribute]]
   }
-  
+
   if (!is.null(format)) {
     if (!is.function(format)) stop("format must be a function!")
     v <- format(v)
   }
-  
-  if (FALSE) {
-    if (is.vector(v) && length(v) == 1) {
-      names(v) <- node$name 
-    } else if (length(v) > 1) {
-      if (!is.null(names(v))) nms <- names(v)
-      else nms = NULL
-      dim(v) <- c(length(v), 1)
-      colnames(v) <- node$name
-      if (!is.null(nms)) rownames(v) <- nms
-    }
-  }  
+
   return (v)
 }
 
@@ -377,17 +417,17 @@ GetObjectAttribute <- function(node, name) {
 }
 
 #' Set a formatter function on a specific node
-#' 
+#'
 #' Formatter functions set on a Node act as a default formatter when printing and using
 #' the \code{\link{Get}} method. The formatter is inherited, meaning that whenever
 #' \code{Get} fetches an attribute from a \code{Node}, it checks on the \code{Node} or
 #' on any of its ancestors whether a formatter is set.
-#' 
+#'
 #' @param node The node on which to set the formatter
 #' @param name The attribute name for which to set the formatter
 #' @param formatFun The formatter, i.e. a function taking a value as an input, and formatting
 #' returning the formatted value
-#' 
+#'
 #' @examples
 #' data(acme)
 #' acme$Set(id = 1:(acme$totalCount))
@@ -396,14 +436,14 @@ GetObjectAttribute <- function(node, name) {
 #' print(acme, "id")
 #' # Calling Get with an explicit formatter will overwrite the default set on the Node:
 #' print(acme, id = acme$Get("id", format = function(x) paste0("id:", x)))
-#' 
+#'
 #' # Or, to avoid formatters, even though you set them on a Node:
 #' print(acme, id = acme$Get("id", format = identity))
-#' 
-#' 
+#'
+#'
 #' @seealso Get
 #' @seealso print.Node
-#' 
+#'
 #' @export
 SetFormat <- function(node, name, formatFun) {
   if (length(attr(node, "formatters")) == 0) attr(node, "formatters") <- list()
@@ -415,21 +455,21 @@ SetFormat <- function(node, name, formatFun) {
 
 
 #' Test whether all node names are unique.
-#' 
+#'
 #' This can be useful for some conversions.
 #' @param node The root \code{Node} of the \code{data.tree} structure to test
-#' 
+#'
 #' @return \code{TRUE} if all \code{Node$name == TRUE} for all nodes in the tree
-#' 
+#'
 #' @examples
 #' data(acme)
 #' AreNamesUnique(acme)
 #' acme$name <- "IT"
 #' AreNamesUnique(acme)
-#' 
+#'
 #' @seealso as.igraph.Node
 #' @export
 AreNamesUnique <- function(node) {
   mynames <- node$Get("name")
   all(duplicated(mynames) == FALSE)
-}
\ No newline at end of file
+}
