@@ -19,6 +19,36 @@ test_that("as.Node.data.frame", {
 })
 
 
+
+test_that("FromDataFrameTable reserved words", {
+
+  pathString <- c("a/b/c/d", "a/b/c/e", "a/f")
+  value <- c("d", "e", "f")
+  
+  df <- data.frame(pathString, value, stringsAsFactors = FALSE)
+
+  #no warn
+  expect_that(tree <- FromDataFrameTable(df, na.rm = TRUE), not(gives_warning()))
+  expect_equal(Get(tree$leaves, "value"), c(d = "d", e = "e", f = "f"))
+  
+  expect_that(tree <- FromDataFrameTable(df, na.rm = TRUE, check = "no-warn"), not(gives_warning()))
+  expect_equal(Get(tree$leaves, "value"), c(d = "d", e = "e", f = "f"))
+
+  #reserved words
+  pathString <- c("name/path/height/count", "name/path/height/e", "name/leaves")
+  value <- c("d", "e", "f")
+  df <- data.frame(pathString, value, stringsAsFactors = FALSE)
+  expect_that(tree <- FromDataFrameTable(df, na.rm = TRUE), gives_warning())
+  expect_equal(Get(tree$leaves, "value"), c(count2 = "d", e = "e", leaves2 = "f"))
+  
+  df <- data.frame(pathString, value, stringsAsFactors = FALSE)
+  expect_that(tree <- FromDataFrameTable(df, na.rm = TRUE, check = "no-warn"), not(gives_warning()))
+  expect_equal(Get(tree$leaves, "value"), c(count2 = "d", e = "e", leaves2 = "f"))
+  
+
+})
+
+
 test_that("as.data.frame.Node", {
   data(acme)
   acmedf <- as.data.frame(acme, 
@@ -153,6 +183,10 @@ test_that("FromDataFrameNetwork order", {
   xN <- FromDataFrameNetwork(x)
   expect_equal(xN$Get("name"), acme$Get("name"))
 
+  x <- x[, c('to', 'from', 'p', 'cost')]
+  xN <- FromDataFrameNetwork(x)
+  expect_equal(xN$Get("name"), acme$Get("name"))
+  
 })
 
 
